@@ -1,4 +1,4 @@
-/* $Id: config.h 2154 2008-10-09 06:00:29Z peter $ */
+/* $Id: config.h 2195 2009-04-10 05:34:14Z peter $ */
 
 #define yasm__splitpath(path, tail)     yasm__splitpath_win(path, tail)
 #define yasm__abspath(path)             yasm__abspath_win(path)
@@ -31,6 +31,9 @@
 /* Define to 1 if you have the <direct.h> header file. */
 /* #undef HAVE_DIRECT_H */
 
+/* Define to 1 if you have the `ftruncate' function. */
+#define HAVE_FTRUNCATE 1
+
 /* Define to 1 if you have the `getcwd' function. */
 #define HAVE_GETCWD 1
 
@@ -116,22 +119,22 @@
 #define PACKAGE_BUGREPORT "bug-yasm@tortall.net"
 
 /* Define to build version of this package. */
-#define PACKAGE_BUILD "2153"
+#define PACKAGE_BUILD "2194"
 
 /* Define to internal version of this package. */
-#define PACKAGE_INTVER "0.7.2"
+#define PACKAGE_INTVER "0.8.0"
 
 /* Define to the full name of this package. */
 #define PACKAGE_NAME "yasm"
 
 /* Define to the full name and version of this package. */
-#define PACKAGE_STRING "yasm 0.7.2"
+#define PACKAGE_STRING "yasm 0.8.0"
 
 /* Define to the one symbol short name of this package. */
 #define PACKAGE_TARNAME "yasm"
 
 /* Define to the version of this package. */
-#define PACKAGE_VERSION "0.7.2"
+#define PACKAGE_VERSION "0.8.0"
 
 /* Define if the C compiler supports function prototypes. */
 #define PROTOTYPES 1
@@ -155,7 +158,7 @@
 #define STDC_HEADERS 1
 
 /* Version number of package */
-#define VERSION "0.7.2"
+#define VERSION "0.8.0"
 
 /* Define if using the dmalloc debugging malloc package */
 /* #undef WITH_DMALLOC */
