@@ -25,7 +25,7 @@
  * POSSIBILITY OF SUCH DAMAGE.
  */
 #include <util.h>
-/*@unused@*/ RCSID("$Id: yasm.c 2143 2008-10-08 07:29:43Z peter $");
+/*@unused@*/ RCSID("$Id: yasm.c 2109 2008-06-08 09:06:05Z peter $");
 
 #include <ctype.h>
 #include <libyasm/compat-queue.h>
@@ -38,6 +38,10 @@
 
 #include "yasm-options.h"
 
+#ifdef CMAKE_BUILD
+#include "yasm-plugin.h"
+#endif
+
 #include "license.c"
 
 /* Preprocess-only buffer size */
@@ -105,6 +109,9 @@ static int opt_ewmsg_handler(char *cmd, /*@null@*/ char *param, int extra);
 static int opt_makedep_handler(char *cmd, /*@null@*/ char *param, int extra);
 static int opt_prefix_handler(char *cmd, /*@null@*/ char *param, int extra);
 static int opt_suffix_handler(char *cmd, /*@null@*/ char *param, int extra);
+#ifdef CMAKE_BUILD
+static int opt_plugin_handler(char *cmd, /*@null@*/ char *param, int extra);
+#endif
 
 static /*@only@*/ char *replace_extension(const char *orig, /*@null@*/
                                           const char *ext, const char *def);
@@ -123,6 +130,7 @@ static void print_yasm_warning(const char *filename, unsigned long line,
                                const char *msg);
 
 static void apply_preproc_builtins(void);
+static void apply_preproc_standard_macros(const yasm_stdmac *stdmacs);
 static void apply_preproc_saved_options(void);
 static void print_list_keyword_desc(const char *name, const char *keyword);
 
@@ -197,6 +205,10 @@ static opt_option options[] =
       N_("append argument to name of all external symbols"), N_("suffix") },
     { 0, "postfix", 1, opt_suffix_handler, 0,
       N_("append argument to name of all external symbols"), N_("suffix") },
+#ifdef CMAKE_BUILD
+    { 'N', "plugin", 1, opt_plugin_handler, 0,
+      N_("load plugin module"), N_("plugin") },
+#endif
 };
 
 /* version message */
@@ -273,13 +285,17 @@ do_preproc_only(void)
             return EXIT_FAILURE;
     }
 
-    /* Pre-process until done */
+    /* Create preprocessor */
     cur_preproc = yasm_preproc_create(cur_preproc_module, in_filename, NULL,
                                       linemap, errwarns);
 
+    /* Apply macros */
     apply_preproc_builtins();
+    apply_preproc_standard_macros(cur_parser_module->stdmacs);
+    apply_preproc_standard_macros(cur_objfmt_module->stdmacs);
     apply_preproc_saved_options();
 
+    /* Pre-process until done */
     if (generate_make_dependencies) {
         size_t totlen;
 
@@ -443,6 +459,8 @@ do_assemble(void)
                                       object->symtab, linemap, errwarns);
 
     apply_preproc_builtins();
+    apply_preproc_standard_macros(cur_parser_module->stdmacs);
+    apply_preproc_standard_macros(cur_objfmt_module->stdmacs);
     apply_preproc_saved_options();
 
     /* Get initial x86 BITS setting from object format */
@@ -567,6 +585,24 @@ main(int argc, char *argv[])
     yasm_gettext_hook = handle_yasm_gettext;
     yasm_errwarn_initialize();
 
+    /* Initialize BitVector (needed for intnum/floatnum). */
+    if (BitVector_Boot() != ErrCode_Ok) {
+        print_error(_("%s: could not initialize BitVector"), _("FATAL"));
+        return EXIT_FAILURE;
+    }
+
+    /* Initialize intnum and floatnum */
+    yasm_intnum_initialize();
+    yasm_floatnum_initialize();
+
+#ifdef CMAKE_BUILD
+    /* Load standard modules */
+    if (!load_plugin("yasmstd")) {
+        print_error(_("%s: could not load standard modules"), _("FATAL"));
+        return EXIT_FAILURE;
+    }
+#endif
+
     /* Initialize parameter storage */
     STAILQ_INIT(&preproc_options);
 
@@ -598,16 +634,6 @@ main(int argc, char *argv[])
             return EXIT_FAILURE;
     }
 
-    /* Initialize BitVector (needed for intnum/floatnum). */
-    if (BitVector_Boot() != ErrCode_Ok) {
-        print_error(_("%s: could not initialize BitVector"), _("FATAL"));
-        return EXIT_FAILURE;
-    }
-
-    /* Initialize intnum and floatnum */
-    yasm_intnum_initialize();
-    yasm_floatnum_initialize();
-
     /* If not already specified, default to bin as the object format. */
     if (!cur_objfmt_module) {
         if (!objfmt_keyword)
@@ -771,6 +797,9 @@ cleanup(yasm_object *object)
 
     if (errfile != stderr && errfile != stdout)
         fclose(errfile);
+#ifdef CMAKE_BUILD
+    unload_plugins();
+#endif
 }
 
 /*
@@ -1149,6 +1178,17 @@ opt_suffix_handler(/*@unused@*/ char *cmd, char *param, /*@unused@*/ int extra)
     return 0;
 }
 
+#ifdef CMAKE_BUILD
+static int
+opt_plugin_handler(/*@unused@*/ char *cmd, char *param,
+                   /*@unused@*/ int extra)
+{
+    if (!load_plugin(param))
+        print_error(_("warning: could not load plugin `%s'"), param);
+    return 0;
+}
+#endif
+
 static void
 apply_preproc_builtins()
 {
@@ -1163,6 +1203,25 @@ apply_preproc_builtins()
     yasm_xfree(predef);
 }
 
+static void
+apply_preproc_standard_macros(const yasm_stdmac *stdmacs)
+{
+    int i, matched;
+
+    if (!stdmacs)
+        return;
+
+    matched = -1;
+    for (i=0; stdmacs[i].parser; i++)
+        if (yasm__strcasecmp(stdmacs[i].parser,
+                             cur_parser_module->keyword) == 0 &&
+            yasm__strcasecmp(stdmacs[i].preproc,
+                             cur_preproc_module->keyword) == 0)
+            matched = i;
+    if (matched >= 0 && stdmacs[matched].macros)
+        yasm_preproc_add_standard(cur_preproc, stdmacs[matched].macros);
+}
+
 static void
 apply_preproc_saved_options()
 {
