@@ -25,7 +25,7 @@
  * POSSIBILITY OF SUCH DAMAGE.
  */
 #include "util.h"
-/*@unused@*/ RCSID("$Id: bytecode.c 1895 2007-07-14 05:31:08Z peter $");
+/*@unused@*/ RCSID("$Id: bytecode.c 2130 2008-10-07 05:38:11Z peter $");
 
 #include "libyasm-stdint.h"
 #include "coretype.h"
@@ -222,6 +222,18 @@ yasm_bc_next_offset(yasm_bytecode *precbc)
     return precbc->offset + precbc->len*precbc->mult_int;
 }
 
+int
+yasm_bc_elem_size(yasm_bytecode *bc)
+{
+    if (!bc->callback) {
+        yasm_internal_error(N_("got empty bytecode in yasm_bc_elem_size"));
+        return 0;
+    } else if (!bc->callback->elem_size)
+        return 0;
+    else
+        return bc->callback->elem_size(bc);
+}
+
 int
 yasm_bc_calc_len(yasm_bytecode *bc, yasm_bc_add_span_func add_span,
                  void *add_span_data)
