@@ -25,7 +25,7 @@
  * POSSIBILITY OF SUCH DAMAGE.
  */
 #include <util.h>
-/*@unused@*/ RCSID("$Id: x86arch.c 2144 2008-10-08 07:33:10Z peter $");
+/*@unused@*/ RCSID("$Id: x86arch.c 2181 2009-03-20 07:36:49Z peter $");
 
 #include <libyasm.h>
 
@@ -68,9 +68,12 @@ x86_create(const char *machine, const char *parser,
     arch_x86->mode_bits = 0;
     arch_x86->force_strict = 0;
     arch_x86->default_rel = 0;
+    arch_x86->nop = X86_NOP_BASIC;
 
     if (yasm__strcasecmp(parser, "nasm") == 0)
         arch_x86->parser = X86_PARSER_NASM;
+    else if (yasm__strcasecmp(parser, "tasm") == 0)
+        arch_x86->parser = X86_PARSER_TASM;
     else if (yasm__strcasecmp(parser, "gas") == 0
              || yasm__strcasecmp(parser, "gnu") == 0)
         arch_x86->parser = X86_PARSER_GAS;
@@ -216,163 +219,227 @@ x86_get_fill(const yasm_arch *arch)
     const yasm_arch_x86 *arch_x86 = (const yasm_arch_x86 *)arch;
 
     /* Fill patterns that GAS uses. */
-    static const unsigned char *fill16[16] = {
-        NULL,                           /* unused                       */
-        (const unsigned char *)
-        "\x90",                         /* 1 - nop                      */
-        (const unsigned char *)
-        "\x89\xf6",                     /* 2 - mov si, si               */
-        (const unsigned char *)
-        "\x8d\x74\x00",                 /* 3 - lea si, [si+byte 0]      */
-        (const unsigned char *)
-        "\x8d\xb4\x00\x00",             /* 4 - lea si, [si+word 0]      */
-        (const unsigned char *)
-        "\x90"                          /* 5 - nop                      */
-        "\x8d\xb4\x00\x00",             /*     lea si, [si+word 0]      */
-        (const unsigned char *)
-        "\x89\xf6"                      /* 6 - mov si, si               */
-        "\x8d\xbd\x00\x00",             /*     lea di, [di+word 0]      */
-        (const unsigned char *)
-        "\x8d\x74\x00"                  /* 7 - lea si, [si+byte 0]      */
-        "\x8d\xbd\x00\x00",             /*     lea di, [di+word 0]      */
-        (const unsigned char *)
-        "\x8d\xb4\x00\x00"              /* 8 - lea si, [si+word 0]      */
-        "\x8d\xbd\x00\x00",             /*     lea di, [di+word 0]      */
-        (const unsigned char *)
-        "\xeb\x07\x90\x90\x90\x90\x90"  /* 9 - jmp $+9; nop fill        */
-        "\x90\x90",
-        (const unsigned char *)
-        "\xeb\x08\x90\x90\x90\x90\x90"  /* 10 - jmp $+10; nop fill      */
-        "\x90\x90\x90",
-        (const unsigned char *)
-        "\xeb\x09\x90\x90\x90\x90\x90"  /* 11 - jmp $+11; nop fill      */
-        "\x90\x90\x90\x90",
-        (const unsigned char *)
-        "\xeb\x0a\x90\x90\x90\x90\x90"  /* 12 - jmp $+12; nop fill      */
-        "\x90\x90\x90\x90\x90",
-        (const unsigned char *)
-        "\xeb\x0b\x90\x90\x90\x90\x90"  /* 13 - jmp $+13; nop fill      */
-        "\x90\x90\x90\x90\x90\x90",
-        (const unsigned char *)
-        "\xeb\x0c\x90\x90\x90\x90\x90"  /* 14 - jmp $+14; nop fill      */
-        "\x90\x90\x90\x90\x90\x90\x90",
-        (const unsigned char *)
-        "\xeb\x0d\x90\x90\x90\x90\x90"  /* 15 - jmp $+15; nop fill      */
-        "\x90\x90\x90\x90\x90\x90\x90\x90"
+    static const unsigned char fill16_1[1] =
+        {0x90};                                 /* 1 - nop */
+    static const unsigned char fill16_2[2] =
+        {0x89, 0xf6};                           /* 2 - mov si, si */
+    static const unsigned char fill16_3[3] =
+        {0x8d, 0x74, 0x00};                     /* 3 - lea si, [si+byte 0] */
+    static const unsigned char fill16_4[4] =
+        {0x8d, 0xb4, 0x00, 0x00};               /* 4 - lea si, [si+word 0] */
+    static const unsigned char fill16_5[5] =
+        {0x90,                                  /* 5 - nop */
+         0x8d, 0xb4, 0x00, 0x00};               /*     lea si, [si+word 0] */
+    static const unsigned char fill16_6[6] =
+        {0x89, 0xf6,                            /* 6 - mov si, si */
+         0x8d, 0xbd, 0x00, 0x00};               /*     lea di, [di+word 0] */
+    static const unsigned char fill16_7[7] =
+        {0x8d, 0x74, 0x00,                      /* 7 - lea si, [si+byte 0] */
+         0x8d, 0xbd, 0x00, 0x00};               /*     lea di, [di+word 0] */
+    static const unsigned char fill16_8[8] =
+        {0x8d, 0xb4, 0x00, 0x00,                /* 8 - lea si, [si+word 0] */
+         0x8d, 0xbd, 0x00, 0x00};               /*     lea di, [di+word 0] */
+    static const unsigned char fill16_9[9] =
+        {0xeb, 0x07, 0x90, 0x90, 0x90, 0x90,    /* 9 - jmp $+9; nop fill */
+         0x90, 0x90, 0x90};
+    static const unsigned char fill16_10[10] =
+        {0xeb, 0x08, 0x90, 0x90, 0x90, 0x90,    /* 10 - jmp $+10; nop fill */
+         0x90, 0x90, 0x90, 0x90};
+    static const unsigned char fill16_11[11] =
+        {0xeb, 0x09, 0x90, 0x90, 0x90, 0x90,    /* 11 - jmp $+11; nop fill */
+         0x90, 0x90, 0x90, 0x90, 0x90};
+    static const unsigned char fill16_12[12] =
+        {0xeb, 0x0a, 0x90, 0x90, 0x90, 0x90,    /* 12 - jmp $+12; nop fill */
+         0x90, 0x90, 0x90, 0x90, 0x90, 0x90};
+    static const unsigned char fill16_13[13] =
+        {0xeb, 0x0b, 0x90, 0x90, 0x90, 0x90,    /* 13 - jmp $+13; nop fill */
+         0x90, 0x90, 0x90, 0x90, 0x90, 0x90, 0x90};
+    static const unsigned char fill16_14[14] =
+        {0xeb, 0x0c, 0x90, 0x90, 0x90, 0x90,    /* 14 - jmp $+14; nop fill */
+         0x90, 0x90, 0x90, 0x90, 0x90, 0x90, 0x90, 0x90};
+    static const unsigned char fill16_15[15] =
+        {0xeb, 0x0d, 0x90, 0x90, 0x90, 0x90,    /* 15 - jmp $+15; nop fill */
+         0x90, 0x90, 0x90, 0x90, 0x90, 0x90, 0x90, 0x90, 0x90};
+    static const unsigned char *fill16[16] =
+    {
+        NULL,      fill16_1,  fill16_2,  fill16_3,
+        fill16_4,  fill16_5,  fill16_6,  fill16_7,
+        fill16_8,  fill16_9,  fill16_10, fill16_11,
+        fill16_12, fill16_13, fill16_14, fill16_15
     };
-    static const unsigned char *fill32[16] = {
-        NULL,                           /* unused                       */
-        (const unsigned char *)
-        "\x90",                         /* 1 - nop                      */
-        (const unsigned char *)
-        "\x89\xf6",                     /* 2 - mov esi, esi             */
-        (const unsigned char *)
-        "\x8d\x76\x00",                 /* 3 - lea esi, [esi+byte 0]    */
-        (const unsigned char *)
-        "\x8d\x74\x26\x00",             /* 4 - lea esi, [esi*1+byte 0]  */
-        (const unsigned char *)
-        "\x90"                          /* 5 - nop                      */
-        "\x8d\x74\x26\x00",             /*     lea esi, [esi*1+byte 0]  */
-        (const unsigned char *)
-        "\x8d\xb6\x00\x00\x00\x00",     /* 6 - lea esi, [esi+dword 0]   */
-        (const unsigned char *)
-        "\x8d\xb4\x26\x00\x00\x00\x00", /* 7 - lea esi, [esi*1+dword 0] */
-        (const unsigned char *)
-        "\x90"                          /* 8 - nop                      */
-        "\x8d\xb4\x26\x00\x00\x00\x00", /*     lea esi, [esi*1+dword 0] */
+
+    static const unsigned char fill32_1[1] =
+        {0x90};                              /* 1 - nop */
+    static const unsigned char fill32_2[2] =
+        {0x66, 0x90};                        /* 2 - xchg ax, ax (o16 nop) */
+    static const unsigned char fill32_3[3] =
+        {0x8d, 0x76, 0x00};                  /* 3 - lea esi, [esi+byte 0] */
+    static const unsigned char fill32_4[4] =
+        {0x8d, 0x74, 0x26, 0x00};            /* 4 - lea esi, [esi*1+byte 0] */
+    static const unsigned char fill32_5[5] =
+        {0x90,                               /* 5 - nop */
+         0x8d, 0x74, 0x26, 0x00};            /*     lea esi, [esi*1+byte 0] */
+    static const unsigned char fill32_6[6] =
+        {0x8d, 0xb6, 0x00, 0x00, 0x00, 0x00};/* 6 - lea esi, [esi+dword 0] */
+    static const unsigned char fill32_7[7] =
+        {0x8d, 0xb4, 0x26, 0x00, 0x00, 0x00, /* 7 - lea esi, [esi*1+dword 0] */
+         0x00};
+    static const unsigned char fill32_8[8] =
+        {0x90,                               /* 8 - nop */
+         0x8d, 0xb4, 0x26, 0x00, 0x00, 0x00, /*     lea esi, [esi*1+dword 0] */
+         0x00};
 #if 0
-        /* GAS uses these */
-        (const unsigned char *)
-        "\x89\xf6"                      /* 9 - mov esi, esi             */
-        "\x8d\xbc\x27\x00\x00\x00\x00", /*     lea edi, [edi*1+dword 0] */
-        (const unsigned char *)
-        "\x8d\x76\x00"                  /* 10 - lea esi, [esi+byte 0]   */
-        "\x8d\xbc\x27\x00\x00\x00\x00", /*      lea edi, [edi+dword 0]  */
-        (const unsigned char *)
-        "\x8d\x74\x26\x00"              /* 11 - lea esi, [esi*1+byte 0] */
-        "\x8d\xbc\x27\x00\x00\x00\x00", /*      lea edi, [edi*1+dword 0]*/
-        (const unsigned char *)
-        "\x8d\xb6\x00\x00\x00\x00"      /* 12 - lea esi, [esi+dword 0]  */
-        "\x8d\xbf\x00\x00\x00\x00",     /*      lea edi, [edi+dword 0]  */
-        (const unsigned char *)
-        "\x8d\xb6\x00\x00\x00\x00"      /* 13 - lea esi, [esi+dword 0]  */
-        "\x8d\xbc\x27\x00\x00\x00\x00", /*      lea edi, [edi*1+dword 0]*/
-        (const unsigned char *)
-        "\x8d\xb4\x26\x00\x00\x00\x00"  /* 14 - lea esi, [esi*1+dword 0]*/
-        "\x8d\xbc\x27\x00\x00\x00\x00", /*      lea edi, [edi*1+dword 0]*/
+    /* GAS uses these */
+    static const unsigned char fill32_9[9] =
+        {0x89, 0xf6,                         /* 9 - mov esi, esi */
+         0x8d, 0xbc, 0x27, 0x00, 0x00, 0x00, /*     lea edi, [edi*1+dword 0] */
+         0x00};
+    static const unsigned char fill32_10[10] =
+        {0x8d, 0x76, 0x00,                   /* 10 - lea esi, [esi+byte 0] */
+         0x8d, 0xbc, 0x27, 0x00, 0x00, 0x00, /*      lea edi, [edi+dword 0] */
+         0x00};
+    static const unsigned char fill32_11[11] =
+        {0x8d, 0x74, 0x26, 0x00,             /* 11 - lea esi, [esi*1+byte 0] */
+         0x8d, 0xbc, 0x27, 0x00, 0x00, 0x00, /*      lea edi, [edi*1+dword 0] */
+         0x00};
+    static const unsigned char fill32_12[12] =
+        {0x8d, 0xb6, 0x00, 0x00, 0x00, 0x00, /* 12 - lea esi, [esi+dword 0] */
+         0x8d, 0xbf, 0x00, 0x00, 0x00, 0x00};/*      lea edi, [edi+dword 0] */
+    static const unsigned char fill32_13[13] =
+        {0x8d, 0xb6, 0x00, 0x00, 0x00, 0x00, /* 13 - lea esi, [esi+dword 0] */
+         0x8d, 0xbc, 0x27, 0x00, 0x00, 0x00, /*      lea edi, [edi*1+dword 0] */
+         0x00};
+    static const unsigned char fill32_14[14] =
+        {0x8d, 0xb4, 0x26, 0x00, 0x00, 0x00, /* 14 - lea esi, [esi*1+dword 0] */
+         0x00,
+         0x8d, 0xbc, 0x27, 0x00, 0x00, 0x00, /*      lea edi, [edi*1+dword 0] */
+         0x00};
 #else
-        /* But on newer processors, these are recommended */
-        (const unsigned char *)
-        "\xeb\x07\x90\x90\x90\x90\x90"  /* 9 - jmp $+9; nop fill        */
-        "\x90\x90",
-        (const unsigned char *)
-        "\xeb\x08\x90\x90\x90\x90\x90"  /* 10 - jmp $+10; nop fill      */
-        "\x90\x90\x90",
-        (const unsigned char *)
-        "\xeb\x09\x90\x90\x90\x90\x90"  /* 11 - jmp $+11; nop fill      */
-        "\x90\x90\x90\x90",
-        (const unsigned char *)
-        "\xeb\x0a\x90\x90\x90\x90\x90"  /* 12 - jmp $+12; nop fill      */
-        "\x90\x90\x90\x90\x90",
-        (const unsigned char *)
-        "\xeb\x0b\x90\x90\x90\x90\x90"  /* 13 - jmp $+13; nop fill      */
-        "\x90\x90\x90\x90\x90\x90",
-        (const unsigned char *)
-        "\xeb\x0c\x90\x90\x90\x90\x90"  /* 14 - jmp $+14; nop fill      */
-        "\x90\x90\x90\x90\x90\x90\x90",
+    /* But on newer processors, these are recommended */
+    static const unsigned char fill32_9[9] =
+        {0xeb, 0x07, 0x90, 0x90, 0x90, 0x90, /* 9 - jmp $+9; nop fill */
+         0x90, 0x90, 0x90};
+    static const unsigned char fill32_10[10] =
+        {0xeb, 0x08, 0x90, 0x90, 0x90, 0x90, /* 10 - jmp $+10; nop fill */
+         0x90, 0x90, 0x90, 0x90};
+    static const unsigned char fill32_11[11] =
+        {0xeb, 0x09, 0x90, 0x90, 0x90, 0x90, /* 11 - jmp $+11; nop fill */
+         0x90, 0x90, 0x90, 0x90, 0x90};
+    static const unsigned char fill32_12[12] =
+        {0xeb, 0x0a, 0x90, 0x90, 0x90, 0x90, /* 12 - jmp $+12; nop fill */
+         0x90, 0x90, 0x90, 0x90, 0x90, 0x90};
+    static const unsigned char fill32_13[13] =
+        {0xeb, 0x0b, 0x90, 0x90, 0x90, 0x90, /* 13 - jmp $+13; nop fill */
+         0x90, 0x90, 0x90, 0x90, 0x90, 0x90, 0x90};
+    static const unsigned char fill32_14[14] =
+        {0xeb, 0x0c, 0x90, 0x90, 0x90, 0x90, /* 14 - jmp $+14; nop fill */
+         0x90, 0x90, 0x90, 0x90, 0x90, 0x90, 0x90, 0x90};
 #endif
-        (const unsigned char *)
-        "\xeb\x0d\x90\x90\x90\x90\x90"  /* 15 - jmp $+15; nop fill      */
-        "\x90\x90\x90\x90\x90\x90\x90\x90"
+    static const unsigned char fill32_15[15] =
+        {0xeb, 0x0d, 0x90, 0x90, 0x90, 0x90, /* 15 - jmp $+15; nop fill */
+         0x90, 0x90, 0x90, 0x90, 0x90, 0x90, 0x90, 0x90, 0x90};
+    static const unsigned char *fill32[16] =
+    {
+        NULL,      fill32_1,  fill32_2,  fill32_3,
+        fill32_4,  fill32_5,  fill32_6,  fill32_7,
+        fill32_8,  fill32_9,  fill32_10, fill32_11,
+        fill32_12, fill32_13, fill32_14, fill32_15
     };
-    static const unsigned char *fill64[16] = {
-        NULL,                           /* unused                       */
-        (const unsigned char *)
-        "\x90",                         /* 1 - nop                      */
-        (const unsigned char *)
-        "\x66\x90",                     /* 2 - o16; nop                 */
-        (const unsigned char *)
-        "\x66\x66\x90",                 /* 3 - o16; o16; nop            */
-        (const unsigned char *)
-        "\x66\x66\x66\x90",             /* 4 - o16; o16; o16; nop       */
-        (const unsigned char *)
-        "\x66\x66\x90\x66\x90",         /* 5 */
-        (const unsigned char *)
-        "\x66\x66\x90\x66\x66\x90",     /* 6 */
-        (const unsigned char *)
-        "\x66\x66\x66\x90\x66\x66\x90", /* 7 */
-        (const unsigned char *)
-        "\x66\x66\x66\x90\x66\x66\x66"  /* 8 */
-        "\x90",
-        (const unsigned char *)
-        "\x66\x66\x90\x66\x66\x90\x66"  /* 9 */
-        "\x66\x90",
-        (const unsigned char *)
-        "\x66\x66\x66\x90\x66\x66\x90"  /* 10 */
-        "\x66\x66\x90",
-        (const unsigned char *)
-        "\x66\x66\x66\x90\x66\x66\x66"  /* 11 */
-        "\x90\x66\x66\x90",
-        (const unsigned char *)
-        "\x66\x66\x66\x90\x66\x66\x66"  /* 12 */
-        "\x90\x66\x66\x66\x90",
-        (const unsigned char *)
-        "\x66\x66\x66\x90\x66\x66\x90"  /* 13 */
-        "\x66\x66\x90\x66\x66\x90",
-        (const unsigned char *)
-        "\x66\x66\x66\x90\x66\x66\x66"  /* 14 */
-        "\x90\x66\x66\x90\x66\x66\x90",
-        (const unsigned char *)
-        "\x66\x66\x66\x90\x66\x66\x66"  /* 15 */
-        "\x90\x66\x66\x66\x90\x66\x66\x90"
+
+    /* Long form nops available on more recent Intel and AMD processors */
+    static const unsigned char fill32new_3[3] =
+        {0x0f, 0x1f, 0x00};                         /* 3 - nop(3) */
+    static const unsigned char fill32new_4[4] =
+        {0x0f, 0x1f, 0x40, 0x00};                   /* 4 - nop(4) */
+    static const unsigned char fill32new_5[5] =
+        {0x0f, 0x1f, 0x44, 0x00, 0x00};             /* 5 - nop(5) */
+    static const unsigned char fill32new_6[6] =
+        {0x66, 0x0f, 0x1f, 0x44, 0x00, 0x00};       /* 6 - nop(6) */
+    static const unsigned char fill32new_7[7] =
+        {0x0f, 0x1f, 0x80, 0x00, 0x00, 0x00, 0x00}; /* 7 - nop(7) */
+    static const unsigned char fill32new_8[8] =
+        {0x0f, 0x1f, 0x84, 0x00, 0x00, 0x00, 0x00,  /* 8 - nop(8) */
+         0x00};
+    static const unsigned char fill32new_9[9] =
+        {0x66, 0x0f, 0x1f, 0x84, 0x00, 0x00, 0x00,  /* 9 - nop(9) */
+         0x00, 0x00};
+
+    /* Longer forms preferred by Intel use repeated o16 prefixes */
+    static const unsigned char fill32intel_10[10] =
+        {0x66, 0x2e, 0x0f, 0x1f, 0x84, 0x00, 0x00,  /* 10 - o16; cs; nop */
+         0x00, 0x00, 0x00};
+    static const unsigned char fill32intel_11[11] =
+        {0x66, 0x66, 0x2e, 0x0f, 0x1f, 0x84, 0x00,  /* 11 - 2x o16; cs; nop */
+         0x00, 0x00, 0x00, 0x00};
+    static const unsigned char fill32intel_12[12] =
+        {0x66, 0x66, 0x66, 0x2e, 0x0f, 0x1f, 0x84,  /* 12 - 3x o16; cs; nop */
+         0x00, 0x00, 0x00, 0x00, 0x00};
+    static const unsigned char fill32intel_13[13] =
+        {0x66, 0x66, 0x66, 0x66, 0x2e, 0x0f, 0x1f,  /* 13 - 4x o16; cs; nop */
+         0x84, 0x00, 0x00, 0x00, 0x00, 0x00};
+    static const unsigned char fill32intel_14[14] =
+        {0x66, 0x66, 0x66, 0x66, 0x66, 0x2e, 0x0f,  /* 14 - 5x o16; cs; nop */
+         0x1f, 0x84, 0x00, 0x00, 0x00, 0x00, 0x00};
+    static const unsigned char fill32intel_15[15] =
+        {0x66, 0x66, 0x66, 0x66, 0x66, 0x66, 0x2e,  /* 15 - 6x o16; cs; nop */
+         0x0f, 0x1f, 0x84, 0x00, 0x00, 0x00, 0x00, 0x00};
+
+    /* Longer forms preferred by AMD use fewer o16 prefixes and no CS prefix;
+     * Source: Software Optimisation Guide for AMD Family 10h
+     * Processors 40546 revision 3.10 February 2009
+     */
+    static const unsigned char fill32amd_10[10] =
+        {0x66, 0x66, 0x0f, 0x1f, 0x84, 0x00, 0x00,  /* 10 - nop(10) */
+         0x00, 0x00, 0x00};
+    static const unsigned char fill32amd_11[11] =
+        {0x0f, 0x1f, 0x44, 0x00, 0x00,              /* 11 - nop(5) */
+         0x66, 0x0f, 0x1f, 0x44, 0x00, 0x00};       /*      nop(6) */
+    static const unsigned char fill32amd_12[12] =
+        {0x66, 0x0f, 0x1f, 0x44, 0x00, 0x00,        /* 12 - nop(6) */
+         0x66, 0x0f, 0x1f, 0x44, 0x00, 0x00};       /*      nop(6) */
+    static const unsigned char fill32amd_13[13] =
+        {0x66, 0x0f, 0x1f, 0x44, 0x00, 0x00,        /* 13 - nop(6) */
+         0x0f, 0x1f, 0x80, 0x00, 0x00, 0x00, 0x00}; /*      nop(7) */
+    static const unsigned char fill32amd_14[14] =
+        {0x0f, 0x1f, 0x80, 0x00, 0x00, 0x00, 0x00,  /* 14 - nop(7) */
+         0x0f, 0x1f, 0x80, 0x00, 0x00, 0x00, 0x00}; /*      nop(7) */
+    static const unsigned char fill32amd_15[15] =
+        {0x0f, 0x1f, 0x80, 0x00, 0x00, 0x00, 0x00,        /* 15 - nop(7) */
+         0x0f, 0x1f, 0x84, 0x00, 0x00, 0x00, 0x00, 0x00}; /*      nop(8) */
+
+    static const unsigned char *fill32_intel[16] =
+    {
+        NULL,           fill32_1,       fill32_2,       fill32new_3,
+        fill32new_4,    fill32new_5,    fill32new_6,    fill32new_7,
+        fill32new_8,    fill32new_9,    fill32intel_10, fill32intel_11,
+        fill32intel_12, fill32intel_13, fill32intel_14, fill32intel_15
     };
+    static const unsigned char *fill32_amd[16] =
+    {
+        NULL,           fill32_1,       fill32_2,       fill32new_3,
+        fill32new_4,    fill32new_5,    fill32new_6,    fill32new_7,
+        fill32new_8,    fill32new_9,    fill32amd_10,   fill32amd_11,
+        fill32amd_12,   fill32amd_13,   fill32amd_14,   fill32amd_15
+    };
+
     switch (arch_x86->mode_bits) {
         case 16:
             return fill16;
         case 32:
-            return fill32;
+            if (arch_x86->nop == X86_NOP_INTEL)
+                return fill32_intel;
+            else if (arch_x86->nop == X86_NOP_AMD)
+                return fill32_amd;
+            else
+                return fill32;
         case 64:
-            return fill64;
+            /* We know long nops are available in 64-bit mode; default to Intel
+             * ones if unspecified (to match GAS behavior).
+             */
+            if (arch_x86->nop == X86_NOP_AMD)
+                return fill32_amd;
+            else
+                return fill32_intel;
         default:
             yasm_error_set(YASM_ERROR_VALUE,
                            N_("Invalid mode_bits in x86_get_fill"));
