@@ -26,7 +26,7 @@
  */
 #include <util.h>
 #include <time.h>
-/*@unused@*/ RCSID("$Id: coff-objfmt.c 2147 2008-10-08 07:39:18Z peter $");
+/*@unused@*/ RCSID("$Id: coff-objfmt.c 2166 2009-01-02 08:33:21Z peter $");
 
 #include <libyasm.h>
 
@@ -234,6 +234,7 @@ static const yasm_bytecode_callback win32_sxdata_bc_callback = {
     win32_sxdata_bc_destroy,
     win32_sxdata_bc_print,
     yasm_bc_finalize_common,
+    NULL,
     win32_sxdata_bc_calc_len,
     yasm_bc_expand_common,
     win32_sxdata_bc_tobytes,
@@ -2198,9 +2199,11 @@ yasm_objfmt_module yasm_coff_LTX_objfmt = {
     "coff",
     "o",
     32,
+    0,
     coff_objfmt_dbgfmt_keywords,
     "null",
     coff_objfmt_directives,
+    NULL,   /* no standard macros */
     coff_objfmt_create,
     coff_objfmt_output,
     coff_objfmt_destroy,
@@ -2219,22 +2222,40 @@ static const char *winXX_objfmt_dbgfmt_keywords[] = {
 
 static const yasm_directive win32_objfmt_directives[] = {
     { ".ident",         "gas",  dir_ident,      YASM_DIR_ANY },
-    { ".export",        "gas",  dir_export,     YASM_DIR_ID_REQUIRED },
     { "ident",          "nasm", dir_ident,      YASM_DIR_ANY },
+    { ".export",        "gas",  dir_export,     YASM_DIR_ID_REQUIRED },
     { "export",         "nasm", dir_export,     YASM_DIR_ID_REQUIRED },
+    { ".safeseh",       "gas",  dir_safeseh,    YASM_DIR_ID_REQUIRED },
     { "safeseh",        "nasm", dir_safeseh,    YASM_DIR_ID_REQUIRED },
     { NULL, NULL, NULL, 0 }
 };
 
+static const char *win32_nasm_stdmac[] = {
+    "%imacro export 1+.nolist",
+    "[export %1]",
+    "%endmacro",
+    "%imacro safeseh 1+.nolist",
+    "[safeseh %1]",
+    "%endmacro",
+    NULL
+};
+
+static const yasm_stdmac win32_objfmt_stdmacs[] = {
+    { "nasm", "nasm", win32_nasm_stdmac },
+    { NULL, NULL, NULL }
+};
+
 /* Define objfmt structure -- see objfmt.h for details */
 yasm_objfmt_module yasm_win32_LTX_objfmt = {
     "Win32",
     "win32",
     "obj",
     32,
+    1,
     winXX_objfmt_dbgfmt_keywords,
     "null",
     win32_objfmt_directives,
+    win32_objfmt_stdmacs,
     win32_objfmt_create,
     coff_objfmt_output,
     coff_objfmt_destroy,
@@ -2248,27 +2269,47 @@ static const yasm_directive win64_objfmt_directives[] = {
     { "ident",          "nasm", dir_ident,      YASM_DIR_ANY },
     { ".export",        "gas",  dir_export,     YASM_DIR_ID_REQUIRED },
     { "export",         "nasm", dir_export,     YASM_DIR_ID_REQUIRED },
+    { ".proc_frame",    "gas",  dir_proc_frame, YASM_DIR_ID_REQUIRED },
     { "proc_frame",     "nasm", dir_proc_frame, YASM_DIR_ID_REQUIRED },
+    { ".pushreg",       "gas",  dir_pushreg,    YASM_DIR_ARG_REQUIRED },
     { "pushreg",        "nasm", dir_pushreg,    YASM_DIR_ARG_REQUIRED },
+    { ".setframe",      "gas",  dir_setframe,   YASM_DIR_ARG_REQUIRED },
     { "setframe",       "nasm", dir_setframe,   YASM_DIR_ARG_REQUIRED },
+    { ".allocstack",    "gas",  dir_allocstack, YASM_DIR_ARG_REQUIRED },
     { "allocstack",     "nasm", dir_allocstack, YASM_DIR_ARG_REQUIRED },
+    { ".savereg",       "gas",  dir_savereg,    YASM_DIR_ARG_REQUIRED },
     { "savereg",        "nasm", dir_savereg,    YASM_DIR_ARG_REQUIRED },
+    { ".savexmm128",    "gas",  dir_savexmm128, YASM_DIR_ARG_REQUIRED },
     { "savexmm128",     "nasm", dir_savexmm128, YASM_DIR_ARG_REQUIRED },
+    { ".pushframe",     "gas",  dir_pushframe,  YASM_DIR_ANY },
     { "pushframe",      "nasm", dir_pushframe,  YASM_DIR_ANY },
+    { ".endprolog",     "gas",  dir_endprolog,  YASM_DIR_ANY },
     { "endprolog",      "nasm", dir_endprolog,  YASM_DIR_ANY },
+    { ".endproc_frame", "gas",  dir_endproc_frame, YASM_DIR_ANY },
     { "endproc_frame",  "nasm", dir_endproc_frame, YASM_DIR_ANY },
     { NULL, NULL, NULL, 0 }
 };
 
+#include "win64-nasm.c"
+#include "win64-gas.c"
+
+static const yasm_stdmac win64_objfmt_stdmacs[] = {
+    { "nasm", "nasm", win64_nasm_stdmac },
+    { "gas", "nasm", win64_gas_stdmac },
+    { NULL, NULL, NULL }
+};
+
 /* Define objfmt structure -- see objfmt.h for details */
 yasm_objfmt_module yasm_win64_LTX_objfmt = {
     "Win64",
     "win64",
     "obj",
     64,
+    1,
     winXX_objfmt_dbgfmt_keywords,
     "null",
     win64_objfmt_directives,
+    win64_objfmt_stdmacs,
     win64_objfmt_create,
     coff_objfmt_output,
     coff_objfmt_destroy,
@@ -2281,9 +2322,11 @@ yasm_objfmt_module yasm_x64_LTX_objfmt = {
     "x64",
     "obj",
     64,
+    1,
     winXX_objfmt_dbgfmt_keywords,
     "null",
     win64_objfmt_directives,
+    win64_objfmt_stdmacs,
     win64_objfmt_create,
     coff_objfmt_output,
     coff_objfmt_destroy,
