@@ -25,7 +25,7 @@
  * POSSIBILITY OF SUCH DAMAGE.
  */
 #include <util.h>
-/*@unused@*/ RCSID("$Id: dbg-objfmt.c 2035 2008-02-09 03:35:07Z peter $");
+/*@unused@*/ RCSID("$Id: dbg-objfmt.c 2166 2009-01-02 08:33:21Z peter $");
 
 #include <libyasm.h>
 
@@ -162,9 +162,11 @@ yasm_objfmt_module yasm_dbg_LTX_objfmt = {
     "dbg",
     "dbg",
     32,
+    0,
     dbg_objfmt_dbgfmt_keywords,
     "null",
     NULL,       /* no directives */
+    NULL,       /* no standard macros */
     dbg_objfmt_create,
     dbg_objfmt_output,
     dbg_objfmt_destroy,
