@@ -26,7 +26,7 @@
  */
 
 #include <util.h>
-/*@unused@*/ RCSID("$Id: elf.c 1976 2007-09-21 16:48:25Z peter $");
+/*@unused@*/ RCSID("$Id: elf.c 2040 2008-02-21 08:57:23Z peter $");
 
 #include <libyasm.h>
 #define YASM_OBJFMT_ELF_INTERNAL
@@ -91,13 +91,24 @@ elf_set_arch(yasm_arch *arch, yasm_symtab *symtab, int bits_pref)
             /* FIXME: misuse of NULL bytecode */
             elf_ssyms[i] = yasm_symtab_define_label(symtab,
                                                     elf_march->ssyms[i].name,
-                                                    NULL, 1, 0);
+                                                    NULL, 0, 0);
         }
     }
 
     return elf_march;
 }
 
+yasm_symrec *
+elf_get_special_sym(const char *name, const char *parser)
+{
+    int i;
+    for (i=0; (unsigned int)i<elf_march->num_ssyms; i++) {
+        if (yasm__strcasecmp(name, elf_march->ssyms[i].name) == 0)
+            return elf_ssyms[i];
+    }
+    return NULL;
+}
+
 /* reloc functions */
 int elf_ssym_has_flag(yasm_symrec *wrt, int flag);
 
@@ -484,6 +495,18 @@ elf_symtab_write_to_file(FILE *f, elf_symtab_head *symtab,
         if (value_intn == NULL)
             value_intn = yasm_intnum_create_uint(entry->value);
 
+        /* If symbol is in a TLS section, force its type to TLS. */
+        if (entry->sym) {
+            yasm_bytecode *precbc;
+            yasm_section *sect;
+            elf_secthead *shead;
+            if (yasm_symrec_get_label(entry->sym, &precbc) &&
+                (sect = yasm_bc_get_section(precbc)) &&
+                (shead = yasm_section_get_data(sect, &elf_section_data)) &&
+                shead->flags & SHF_TLS) {
+                entry->type = STT_TLS;
+            }
+        }
 
         if (!elf_march->write_symtab_entry || !elf_march->symtab_entry_size)
             yasm_internal_error(N_("Unsupported machine for ELF output"));
