@@ -91,7 +91,7 @@
 */
 
 #include <util.h>
-/*@unused@*/ RCSID("$Id: macho-objfmt.c 2143 2008-10-08 07:29:43Z peter $");
+/*@unused@*/ RCSID("$Id: macho-objfmt.c 2166 2009-01-02 08:33:21Z peter $");
 
 #include <libyasm.h>
 
@@ -978,6 +978,7 @@ macho_objfmt_calc_sectsize(yasm_section *sect, /*@null@ */ void *d)
     /*@null@ */ macho_objfmt_output_info *info =
         (macho_objfmt_output_info *) d;
     /*@dependent@ *//*@null@ */ macho_section_data *msd;
+    unsigned long align;
 
     assert(info != NULL);
     msd = yasm_section_get_data(sect, &macho_section_data_cb);
@@ -994,6 +995,16 @@ macho_objfmt_calc_sectsize(yasm_section *sect, /*@null@ */ void *d)
     msd->vmoff = info->vmsize;
     info->vmsize += msd->size;
 
+    /* align both start and end of section */
+    align = yasm_section_get_align(sect);
+    if (align != 0) {
+        unsigned long delta = msd->vmoff % align;
+        if (delta > 0) {
+            msd->vmoff += align - delta;
+            info->vmsize += delta;
+        }
+    }
+
     return 0;
 }
 
@@ -1544,9 +1555,11 @@ yasm_objfmt_module yasm_macho_LTX_objfmt = {
     "macho",
     "o",
     32,
+    0,
     macho_objfmt_dbgfmt_keywords,
     "null",
     NULL,   /* no directives */
+    NULL,   /* no standard macros */
     macho_objfmt_create,
     macho_objfmt_output,
     macho_objfmt_destroy,
@@ -1560,9 +1573,11 @@ yasm_objfmt_module yasm_macho32_LTX_objfmt = {
     "macho32",
     "o",
     32,
+    0,
     macho_objfmt_dbgfmt_keywords,
     "null",
     NULL,   /* no directives */
+    NULL,   /* no standard macros */
     macho32_objfmt_create,
     macho_objfmt_output,
     macho_objfmt_destroy,
@@ -1576,9 +1591,11 @@ yasm_objfmt_module yasm_macho64_LTX_objfmt = {
     "macho64",
     "o",
     64,
+    0,
     macho_objfmt_dbgfmt_keywords,
     "null",
     NULL,   /* no directives */
+    NULL,   /* no standard macros */
     macho64_objfmt_create,
     macho_objfmt_output,
     macho_objfmt_destroy,
