@@ -25,7 +25,7 @@
  * POSSIBILITY OF SUCH DAMAGE.
  */
 #include <util.h>
-/*@unused@*/ RCSID("$Id: rdf-objfmt.c 2143 2008-10-08 07:29:43Z peter $");
+/*@unused@*/ RCSID("$Id: rdf-objfmt.c 2166 2009-01-02 08:33:21Z peter $");
 
 #include <libyasm.h>
 
@@ -1058,15 +1058,32 @@ static const yasm_directive rdf_objfmt_directives[] = {
     { NULL, NULL, NULL, 0 }
 };
 
+static const char *rdf_nasm_stdmac[] = {
+    "%imacro library 1+.nolist",
+    "[library %1]",
+    "%endmacro",
+    "%imacro module 1+.nolist",
+    "[module %1]",
+    "%endmacro",
+    NULL
+};
+
+static const yasm_stdmac rdf_objfmt_stdmacs[] = {
+    { "nasm", "nasm", rdf_nasm_stdmac },
+    { NULL, NULL, NULL }
+};
+
 /* Define objfmt structure -- see objfmt.h for details */
 yasm_objfmt_module yasm_rdf_LTX_objfmt = {
     "Relocatable Dynamic Object File Format (RDOFF) v2.0",
     "rdf",
     "rdf",
     32,
+    0,
     rdf_objfmt_dbgfmt_keywords,
     "null",
     rdf_objfmt_directives,
+    rdf_objfmt_stdmacs,
     rdf_objfmt_create,
     rdf_objfmt_output,
     rdf_objfmt_destroy,
