@@ -28,10 +28,11 @@
  * POSSIBILITY OF SUCH DAMAGE.
  */
 #include <util.h>
-RCSID("$Id: gas-parse.c 2035 2008-02-09 03:35:07Z peter $");
+RCSID("$Id: gas-parse.c 2169 2009-01-02 20:46:57Z peter $");
 
 #include <libyasm.h>
 
+#include <ctype.h>
 #include <limits.h>
 #include <math.h>
 
@@ -44,7 +45,8 @@ typedef struct dir_lookup {
     enum gas_parser_state newstate;
 } dir_lookup;
 
-static yasm_bytecode * cpp_line_marker(yasm_parser_gas *parser_gas);
+static void cpp_line_marker(yasm_parser_gas *parser_gas);
+static void nasm_line_marker(yasm_parser_gas *parser_gas);
 static yasm_bytecode *parse_instr(yasm_parser_gas *parser_gas);
 static int parse_dirvals(yasm_parser_gas *parser_gas, yasm_valparamhead *vps);
 static int parse_datavals(yasm_parser_gas *parser_gas, yasm_datavalhead *dvs);
@@ -107,14 +109,9 @@ destroy_curtok_(yasm_parser_gas *parser_gas)
             break;
         case ID:
         case LABEL:
-            yasm_xfree(curval.str_val);
-            break;
         case STRING:
             yasm_xfree(curval.str.contents);
             break;
-        case INSN:
-            yasm_bc_destroy(curval.bc);
-            break;
         default:
             break;
     }
@@ -155,8 +152,6 @@ expect_(yasm_parser_gas *parser_gas, int token)
         case INTNUM:            str = "integer"; break;
         case FLTNUM:            str = "floating point value"; break;
         case STRING:            str = "string"; break;
-        case INSN:              str = "instruction"; break;
-        case PREFIX:            str = "instruction prefix"; break;
         case REG:               str = "register"; break;
         case REGGROUP:          str = "register group"; break;
         case SEGREG:            str = "segment register"; break;
@@ -204,7 +199,6 @@ parse_line(yasm_parser_gas *parser_gas)
                 return dir->handler(parser_gas, dir->param);
             }
 
-            parser_gas->state = INSTDIR;
             get_next_token(); /* ID */
             if (curtok == ':') {
                 /* Label */
@@ -248,9 +242,14 @@ parse_line(yasm_parser_gas *parser_gas)
             define_label(parser_gas, LABEL_val, 0);
             get_next_token(); /* LABEL */
             return parse_line(parser_gas);
-        case LINE_MARKER:
+        case CPP_LINE_MARKER:
             get_next_token();
-            return cpp_line_marker(parser_gas);
+            cpp_line_marker(parser_gas);
+            return NULL;
+        case NASM_LINE_MARKER:
+            get_next_token();
+            nasm_line_marker(parser_gas);
+            return NULL;
         default:
             yasm_error_set(YASM_ERROR_SYNTAX,
                 N_("label or instruction expected at start of line"));
@@ -268,10 +267,10 @@ parse_line(yasm_parser_gas *parser_gas)
     without adding a filter to the input before passing it to cpp.
 
     This function is only called if the preprocessor was 'cpp', since the
-    LINE_MARKER token isn't generated for any other preprocessor. With any other
-    preprocessor, anything after a '#' is always treated as a comment.
+    CPP_LINE_MARKER token isn't generated for any other preprocessor. With any
+    other preprocessor, anything after a '#' is always treated as a comment.
 */
-static yasm_bytecode *
+static void
 cpp_line_marker(yasm_parser_gas *parser_gas)
 {
     yasm_valparamhead vps;
@@ -285,14 +284,14 @@ cpp_line_marker(yasm_parser_gas *parser_gas)
         while (curtok != '\n')
             get_next_token();
 
-        return NULL;
+        return;
     }
 
     if (yasm_intnum_sign(INTNUM_val) < 0) {
         get_next_token(); /* INTNUM */
         yasm_error_set(YASM_ERROR_SYNTAX,
                        N_("line number is negative"));
-        return NULL;
+        return;
     }
 
     line = yasm_intnum_get_uint(INTNUM_val);
@@ -315,7 +314,7 @@ cpp_line_marker(yasm_parser_gas *parser_gas)
         while (curtok != '\n')
             get_next_token();
 
-        return NULL;
+        return;
     }
 
     filename = STRING_val.contents;
@@ -339,7 +338,8 @@ cpp_line_marker(yasm_parser_gas *parser_gas)
         yasm_object_directive(p_object, ".file", "gas", &vps, NULL, cur_line);
 
         yasm_vps_delete(&vps);
-    }
+    } else
+        yasm_xfree(filename);
 
     /* Skip flags. */
     while (1) {
@@ -348,18 +348,110 @@ cpp_line_marker(yasm_parser_gas *parser_gas)
                 break;
 
             case '\n':
-                return NULL;
+                return;
 
             default:
                 yasm_error_set(YASM_ERROR_SYNTAX,
                     N_("junk at end of cpp line marker"));
-                return NULL;
+                return;
         }
         get_next_token();
     }
+}
 
-    /* Never reached. */
-    return NULL;
+/*
+    Handle line markers generated by the nasm preproc.
+
+    We expect a positive integer (line) followed by a plus sign, followed by
+    another positive integer, followed by a string (filename).
+
+    This function is only called if the preprocessor was 'nasm', since the
+    NASM_LINE_MARKER token isn't generated for any other preprocessor.
+*/
+static void
+nasm_line_marker(yasm_parser_gas *parser_gas)
+{
+    yasm_valparamhead vps;
+    yasm_valparam *vp;
+    unsigned long line, incr;
+    char *filename;
+
+    /* Line number. */
+    if (!expect(INTNUM)) return;
+
+    if (yasm_intnum_sign(INTNUM_val) < 0) {
+        get_next_token(); /* INTNUM */
+        yasm_error_set(YASM_ERROR_SYNTAX,
+                       N_("line number is negative"));
+        return;
+    }
+
+    line = yasm_intnum_get_uint(INTNUM_val);
+
+    /*
+        Set to (line - 1) since the directive indicates that the *next* line
+        will have the number given.
+
+        cpp should never produce line=0, but the if keeps us safe just incase.
+    */
+    if (line != 0)
+        line--;
+
+    yasm_intnum_destroy(INTNUM_val);
+    get_next_token(); /* INTNUM */
+
+    if (!expect('+')) return;
+    get_next_token(); /* + */
+
+    /* Line number increment. */
+    if (!expect(INTNUM)) return;
+
+    if (yasm_intnum_sign(INTNUM_val) < 0) {
+        get_next_token(); /* INTNUM */
+        yasm_error_set(YASM_ERROR_SYNTAX,
+                       N_("line increment is negative"));
+        return;
+    }
+
+    incr = yasm_intnum_get_uint(INTNUM_val);
+    yasm_intnum_destroy(INTNUM_val);
+
+    /* File name is not in quotes, so need to switch to a different tokenizer
+     * state.
+     */
+    parser_gas->state = NASM_FILENAME;
+    get_next_token(); /* INTNUM */
+    if (!expect(STRING)) {
+        parser_gas->state = INITIAL;
+        return;
+    }
+
+    filename = STRING_val.contents;
+
+    /* Set linemap. */
+    yasm_linemap_set(parser_gas->linemap, filename, line, incr);
+
+    /*
+        The first line marker in the file (which should be on the first line
+        of the file) will give us the name of the source file. This information
+        needs to be passed on to the debug format module.
+    */
+    if (parser_gas->seen_line_marker == 0) {
+        parser_gas->seen_line_marker = 1;
+
+        yasm_vps_initialize(&vps);
+        vp = yasm_vp_create_string(NULL, filename);
+        yasm_vps_append(&vps, vp);
+
+        yasm_object_directive(p_object, ".file", "gas", &vps, NULL, cur_line);
+
+        yasm_vps_delete(&vps);
+    } else
+        yasm_xfree(filename);
+
+    /* We need to poke back on the \n that was consumed by the tokenizer */
+    parser_gas->peek_token = '\n';
+    get_next_token();
 }
 
 /* Line directive */
@@ -840,15 +932,38 @@ static yasm_bytecode *
 parse_instr(yasm_parser_gas *parser_gas)
 {
     yasm_bytecode *bc;
+    char *id;
+    size_t id_len;
+    uintptr_t prefix;
 
-    switch (curtok) {
-        case INSN:
+    if (curtok != ID)
+        return NULL;
+
+    id = ID_val;
+    id_len = ID_len;
+
+    /* instructions/prefixes must start with a letter */
+    if (!isalpha(id[0]))
+        return NULL;
+
+    /* check to be sure it's not a label or equ */
+    get_peek_token(parser_gas);
+    if (parser_gas->peek_token == ':' || parser_gas->peek_token == '=')
+        return NULL;
+
+    switch (yasm_arch_parse_check_insnprefix
+            (p_object->arch, ID_val, ID_len, cur_line, &bc, &prefix)) {
+        case YASM_ARCH_INSN:
         {
             yasm_insn *insn;
-            bc = INSN_val;
+
+            /* Propagate errors in case we got a warning from the arch */
+            yasm_errwarn_propagate(parser_gas->errwarns, cur_line);
+
             insn = yasm_bc_get_insn(bc);
 
-            get_next_token();
+            yasm_xfree(id);
+            get_next_token();   /* ID */
             if (is_eol())
                 return bc;      /* no operands */
 
@@ -873,25 +988,32 @@ parse_instr(yasm_parser_gas *parser_gas)
             }
             return bc;
         }
-        case PREFIX:
-        {
-            uintptr_t prefix = PREFIX_val;
-            get_next_token(); /* PREFIX */
+        case YASM_ARCH_PREFIX:
+            /* Propagate errors in case we got a warning from the arch */
+            yasm_errwarn_propagate(parser_gas->errwarns, cur_line);
+
+            yasm_xfree(id);
+            get_next_token();   /* ID */
             bc = parse_instr(parser_gas);
             if (!bc)
                 bc = yasm_arch_create_empty_insn(p_object->arch, cur_line);
             yasm_insn_add_prefix(yasm_bc_get_insn(bc), prefix);
             return bc;
-        }
-        case SEGREG:
-        {
-            uintptr_t segreg = SEGREG_val;
-            get_next_token(); /* SEGREG */
+        default:
+            break;
+    }
+
+    /* Check for segment register used as prefix */
+    switch (yasm_arch_parse_check_regtmod(p_object->arch, ID_val, ID_len,
+                                          &prefix)) {
+        case YASM_ARCH_SEGREG:
+            yasm_xfree(id);
+            get_next_token();   /* ID */
             bc = parse_instr(parser_gas);
             if (!bc)
                 bc = yasm_arch_create_empty_insn(p_object->arch, cur_line);
-            yasm_insn_add_seg_prefix(yasm_bc_get_insn(bc), segreg);
-        }
+            yasm_insn_add_seg_prefix(yasm_bc_get_insn(bc), prefix);
+            return bc;
         default:
             return NULL;
     }
@@ -928,6 +1050,11 @@ parse_dirvals(yasm_parser_gas *parser_gas, yasm_valparamhead *vps)
                 vp = yasm_vp_create_string(NULL, STRING_val.contents);
                 get_next_token(); /* STRING */
                 break;
+            case REG:
+                e = p_expr_new_ident(yasm_expr_reg(REG_val));
+                vp = yasm_vp_create_expr(NULL, e);
+                get_next_token(); /* REG */
+                break;
             case '@':
                 /* XXX: is throwing it away *really* the right thing? */
                 get_next_token(); /* @ */
@@ -1363,7 +1490,6 @@ parse_expr2(yasm_parser_gas *parser_gas)
                 if (!expect(ID)) {
                     yasm_error_set(YASM_ERROR_SYNTAX,
                                    N_("expected identifier after `@'"));
-                    yasm_xfree(name);
                     return NULL;
                 }
                 wrt = yasm_objfmt_get_special_sym(p_object, ID_val, "gas");
@@ -1549,59 +1675,59 @@ gas_parser_dir_fill(yasm_parser_gas *parser_gas, /*@only@*/ yasm_expr *repeat,
 
 static dir_lookup dirs_static[] = {
     /* FIXME: Whether this is power-of-two or not depends on arch and objfmt. */
-    {".align",      dir_align,  0,  INSTDIR},
-    {".p2align",    dir_align,  1,  INSTDIR},
-    {".balign",     dir_align,  0,  INSTDIR},
-    {".org",        dir_org,    0,  INSTDIR},
+    {".align",      dir_align,  0,  INITIAL},
+    {".p2align",    dir_align,  1,  INITIAL},
+    {".balign",     dir_align,  0,  INITIAL},
+    {".org",        dir_org,    0,  INITIAL},
     /* data visibility directives */
-    {".local",      dir_local,  0,  INSTDIR},
-    {".comm",       dir_comm,   0,  INSTDIR},
-    {".lcomm",      dir_comm,   1,  INSTDIR},
+    {".local",      dir_local,  0,  INITIAL},
+    {".comm",       dir_comm,   0,  INITIAL},
+    {".lcomm",      dir_comm,   1,  INITIAL},
     /* integer data declaration directives */
-    {".byte",       dir_data,   1,  INSTDIR},
-    {".2byte",      dir_data,   2,  INSTDIR},
-    {".4byte",      dir_data,   4,  INSTDIR},
-    {".8byte",      dir_data,   8,  INSTDIR},
-    {".16byte",     dir_data,   16, INSTDIR},
+    {".byte",       dir_data,   1,  INITIAL},
+    {".2byte",      dir_data,   2,  INITIAL},
+    {".4byte",      dir_data,   4,  INITIAL},
+    {".8byte",      dir_data,   8,  INITIAL},
+    {".16byte",     dir_data,   16, INITIAL},
     /* TODO: These should depend on arch */
-    {".short",      dir_data,   2,  INSTDIR},
-    {".int",        dir_data,   4,  INSTDIR},
-    {".long",       dir_data,   4,  INSTDIR},
-    {".hword",      dir_data,   2,  INSTDIR},
-    {".quad",       dir_data,   8,  INSTDIR},
-    {".octa",       dir_data,   16, INSTDIR},
+    {".short",      dir_data,   2,  INITIAL},
+    {".int",        dir_data,   4,  INITIAL},
+    {".long",       dir_data,   4,  INITIAL},
+    {".hword",      dir_data,   2,  INITIAL},
+    {".quad",       dir_data,   8,  INITIAL},
+    {".octa",       dir_data,   16, INITIAL},
     /* XXX: At least on x86, this is 2 bytes */
-    {".value",      dir_data,   2,  INSTDIR},
+    {".value",      dir_data,   2,  INITIAL},
     /* ASCII data declaration directives */
-    {".ascii",      dir_ascii,  0,  INSTDIR},   /* no terminating zero */
-    {".asciz",      dir_ascii,  1,  INSTDIR},   /* add terminating zero */
-    {".string",     dir_ascii,  1,  INSTDIR},   /* add terminating zero */
+    {".ascii",      dir_ascii,  0,  INITIAL},   /* no terminating zero */
+    {".asciz",      dir_ascii,  1,  INITIAL},   /* add terminating zero */
+    {".string",     dir_ascii,  1,  INITIAL},   /* add terminating zero */
     /* LEB128 integer data declaration directives */
-    {".sleb128",    dir_leb128, 1,  INSTDIR},   /* signed */
-    {".uleb128",    dir_leb128, 0,  INSTDIR},   /* unsigned */
+    {".sleb128",    dir_leb128, 1,  INITIAL},   /* signed */
+    {".uleb128",    dir_leb128, 0,  INITIAL},   /* unsigned */
     /* floating point data declaration directives */
-    {".float",      dir_data,   4,  INSTDIR},
-    {".single",     dir_data,   4,  INSTDIR},
-    {".double",     dir_data,   8,  INSTDIR},
-    {".tfloat",     dir_data,   10, INSTDIR},
+    {".float",      dir_data,   4,  INITIAL},
+    {".single",     dir_data,   4,  INITIAL},
+    {".double",     dir_data,   8,  INITIAL},
+    {".tfloat",     dir_data,   10, INITIAL},
     /* section directives */
     {".bss",        dir_bss_section,    0,  INITIAL},
     {".data",       dir_data_section,   0,  INITIAL},
     {".text",       dir_text_section,   0,  INITIAL},
     {".section",    dir_section,        0, SECTION_DIRECTIVE},
     /* macro directives */
-    {".rept",       dir_rept,   0,  INSTDIR},
-    {".endr",       dir_endr,   0,  INSTDIR},
+    {".rept",       dir_rept,   0,  INITIAL},
+    {".endr",       dir_endr,   0,  INITIAL},
     /* empty space/fill directives */
-    {".skip",       dir_skip,   0,  INSTDIR},
-    {".space",      dir_skip,   0,  INSTDIR},
-    {".fill",       dir_fill,   0,  INSTDIR},
-    {".zero",       dir_zero,   0,  INSTDIR},
+    {".skip",       dir_skip,   0,  INITIAL},
+    {".space",      dir_skip,   0,  INITIAL},
+    {".fill",       dir_fill,   0,  INITIAL},
+    {".zero",       dir_zero,   0,  INITIAL},
     /* other directives */
-    {".equ",        dir_equ,    0,  INSTDIR},
-    {".file",       dir_file,   0,  INSTDIR},
-    {".line",       dir_line,   0,  INSTDIR},
-    {".set",        dir_equ,    0,  INSTDIR}
+    {".equ",        dir_equ,    0,  INITIAL},
+    {".file",       dir_file,   0,  INITIAL},
+    {".line",       dir_line,   0,  INITIAL},
+    {".set",        dir_equ,    0,  INITIAL}
 };
 
 static void
@@ -1619,7 +1745,7 @@ gas_parser_parse(yasm_parser_gas *parser_gas)
     word.name = ".word";
     word.handler = dir_data;
     word.param = yasm_arch_wordsize(p_object->arch)/8;
-    word.newstate = INSTDIR;
+    word.newstate = INITIAL;
 
     /* Create directive lookup */
     parser_gas->dirs = HAMT_create(1, yasm_internal_error_);
