@@ -25,7 +25,7 @@
  * POSSIBILITY OF SUCH DAMAGE.
  */
 #include <util.h>
-/*@unused@*/ RCSID("$Id: nasm-parser.c 2007 2007-11-03 04:27:35Z peter $");
+/*@unused@*/ RCSID("$Id: nasm-parser.c 2167 2009-01-02 08:36:09Z peter $");
 
 #include <libyasm.h>
 
@@ -33,12 +33,13 @@
 
 
 static void
-nasm_parser_do_parse(yasm_object *object, yasm_preproc *pp,
-                     int save_input, yasm_linemap *linemap,
-                     yasm_errwarns *errwarns)
+nasm_do_parse(yasm_object *object, yasm_preproc *pp, int save_input,
+              yasm_linemap *linemap, yasm_errwarns *errwarns, int tasm)
 {
     yasm_parser_nasm parser_nasm;
 
+    parser_nasm.tasm = tasm;
+
     parser_nasm.object = object;
     parser_nasm.linemap = linemap;
 
@@ -62,9 +63,6 @@ nasm_parser_do_parse(yasm_object *object, yasm_preproc *pp,
 
     parser_nasm.state = INITIAL;
 
-    /* yacc debugging, needs YYDEBUG set in bison.y.in to work */
-    /* nasm_parser_debug = 1; */
-
     nasm_parser_parse(&parser_nasm);
 
     /*yasm_scanner_delete(&parser_nasm.s);*/
@@ -77,6 +75,16 @@ nasm_parser_do_parse(yasm_object *object, yasm_preproc *pp,
     yasm_symtab_parser_finalize(object->symtab, 0, errwarns);
 }
 
+static void
+nasm_parser_do_parse(yasm_object *object, yasm_preproc *pp,
+                     int save_input, yasm_linemap *linemap,
+                     yasm_errwarns *errwarns)
+{
+    nasm_do_parse(object, pp, save_input, linemap, errwarns, 0);
+}
+
+#include "nasm-macros.c"
+
 /* Define valid preprocessors to use with this parser */
 static const char *nasm_parser_preproc_keywords[] = {
     "raw",
@@ -84,11 +92,49 @@ static const char *nasm_parser_preproc_keywords[] = {
     NULL
 };
 
+static const yasm_stdmac nasm_parser_stdmacs[] = {
+    { "nasm", "nasm", nasm_standard_mac },
+    { NULL, NULL, NULL }
+};
+
 /* Define parser structure -- see parser.h for details */
 yasm_parser_module yasm_nasm_LTX_parser = {
     "NASM-compatible parser",
     "nasm",
     nasm_parser_preproc_keywords,
     "nasm",
+    nasm_parser_stdmacs,
     nasm_parser_do_parse
 };
+
+static void
+tasm_parser_do_parse(yasm_object *object, yasm_preproc *pp,
+                     int save_input, yasm_linemap *linemap,
+                     yasm_errwarns *errwarns)
+{
+    yasm_symtab_set_case_sensitive(object->symtab, 0);
+    yasm_warn_disable(YASM_WARN_IMPLICIT_SIZE_OVERRIDE);
+    nasm_do_parse(object, pp, save_input, linemap, errwarns, 1);
+}
+
+/* Define valid preprocessors to use with this parser */
+static const char *tasm_parser_preproc_keywords[] = {
+    "raw",
+    "tasm",
+    NULL
+};
+
+static const yasm_stdmac tasm_parser_stdmacs[] = {
+    { "tasm", "tasm", nasm_standard_mac },
+    { NULL, NULL, NULL }
+};
+
+/* Define parser structure -- see parser.h for details */
+yasm_parser_module yasm_tasm_LTX_parser = {
+    "TASM-compatible parser",
+    "tasm",
+    tasm_parser_preproc_keywords,
+    "tasm",
+    tasm_parser_stdmacs,
+    tasm_parser_do_parse
+};
