@@ -1,4 +1,4 @@
-/* $Id: nasm-parser.h 2045 2008-03-27 05:35:36Z peter $
+/* $Id: nasm-parser.h 2167 2009-01-02 08:36:09Z peter $
  * NASM-compatible parser header file
  *
  *  Copyright (C) 2002-2007  Peter Johnson
@@ -38,11 +38,14 @@ enum tokentype {
     FILENAME,
     STRING,
     SIZE_OVERRIDE,
+    OFFSET,
     DECLARE_DATA,
     RESERVE_SPACE,
+    LABEL,
     INCBIN,
     EQU,
     TIMES,
+    DUP,
     SEG,
     WRT,
     ABS,
@@ -56,6 +59,8 @@ enum tokentype {
     TARGETMOD,
     LEFT_OP,
     RIGHT_OP,
+    LOW,
+    HIGH,
     SIGNDIV,
     SIGNMOD,
     START_SECTION_ID,
@@ -82,7 +87,7 @@ typedef union {
 #define YYSTYPE yystype
 
 typedef struct yasm_parser_nasm {
-    int debug;
+    int tasm;
 
     /*@only@*/ yasm_object *object;
 
