@@ -103,7 +103,7 @@ cpp_build_cmdline(yasm_preproc_cpp *pp, const char *extra)
     cmdline = p = yasm_xmalloc(strlen(CPP_PROG)+CMDLINE_SIZE);
     limit = p + CMDLINE_SIZE;
     strcpy(p, CPP_PROG);
-    p += 3;
+    p += strlen(CPP_PROG);
 
     arg = TAILQ_FIRST(&pp->cpp_args);
 
@@ -377,6 +377,12 @@ cpp_preproc_define_builtin(yasm_preproc *preproc, const char *macronameval)
     cpp_preproc_predefine_macro(preproc, macronameval);
 }
 
+static void
+cpp_preproc_add_standard(yasm_preproc *preproc, const char **macros)
+{
+    /* TODO */
+}
+
 /*******************************************************************************
     Preprocessor module object.
 *******************************************************************************/
@@ -391,5 +397,6 @@ yasm_preproc_module yasm_cpp_LTX_preproc = {
     cpp_preproc_add_include_file,
     cpp_preproc_predefine_macro,
     cpp_preproc_undefine_macro,
-    cpp_preproc_define_builtin
+    cpp_preproc_define_builtin,
+    cpp_preproc_add_standard
 };
